#ifndef PICTYPES_H
#define PICTYPES_H

/*
 * type [FL]: 	Describes the family of programming algorithms used.
 * 	EPROM  	Eprom devices
 * 	FLASH1 	Early flash devices with only Erase-Program
 * 	FLASH2 	later flash with erase-prog or program only. 
 * 	       	Note may be variant bulk erase algs.
 *             	These devices will program faster when more than 1/2 the locations are
 *             	changing, if you erase before programming. eg F84A,F87X,F62X
 * 	FLASH3 	flash with program only and bulk erase(F7X), externally timed
 *		Always forced to bulk erase before programming.  eg F7X
 * 	FLASH4 	12F675 etc, internally timed, bulk erase (eg. 12F675)
 * 	FLASH5 	Externally timed flash (F818/9/F87xA) but program only paged 
 * 	       	ChipErase is different to FLASH3 (also bulk erase before programming)
 *
 * page_size [PS] size of program memory pages. (F7X, F818/9)
 *                this is not yet used in any programming algorithm except
 *                for the EEPROMs
 * prg_dly	  time for 1 program pulse in us. For flash it is MAX time.
 *		  for all flash devices, bulk erase time is 10ms set in erase routine
 *		  Should be the prg time for a program only cycle. An erase-program
 *		  cycle is twice this.
 * n_tries  [N]	  maximum number of tries before abandoning programming attempts
 * n_reburn [RE]  number of reburn or over-program pulses=Nactual*RE
 * prog_size, eeprom_size [PM,DM] program and data(EEPROM) memory sizes
 * --------------------------------------------------*/

// name, type, core_bits, page_size, prg_dly, n_tries, n_reburn, 
// prog_size, eeprom_size, dev_id, rev_bits

chip_info chips[]= {
{"12C508"  , EPROM , 12,  1,   100,  8, 11,  512,    0, 0, 0},
{"12C508A" , EPROM , 12,  1,   100,  8, 11,  512,    0, 0, 0},
{"12C509"  , EPROM , 12,  1,   100,  8, 11, 1024,    0, 0, 0},
{"12C509A" , EPROM , 12,  1,   100,  8, 11, 1024,    0, 0, 0},
{"12CE518" , EPROM , 12,  1,   100,  8, 11,  512,    0, 0, 0},
{"12CE519" , EPROM , 12,  1,   100,  8, 11, 1024,    0, 0, 0},
{"12C671"  , EPROM , 14,  1,   100, 25,  3, 1024,    0, 0, 0},
{"12C672"  , EPROM , 14,  1,   100, 25,  3, 2048,    0, 0, 0},
{"12CE673" , EPROM , 14,  1,   100, 25,  3, 1024,   16, 0, 0},
{"12CE674" , EPROM , 14,  1,   100, 25,  3, 2048,   16, 0, 0},
{"12F629"  , FLASH4, 14,  1,  6000,  1,  0, 1024,  128, 0x7c, 5},
{"12F675"  , FLASH4, 14,  1,  6000,  1,  0, 1024,  128, 0x7e, 5},
{"12F635"  , FLASH4, 14,  1,  6000,  1,  0, 1024,  128, 0x7d, 5},
{"16F636"  , FLASH4, 14,  1,  6000,  1,  0, 2048,  256, 0x85, 5},
{"12F683"  , FLASH4, 14,  1,  6000,  1,  0, 2048,  256, 0x23, 5},
{"16F684"  , FLASH4, 14,  1,  6000,  1,  0, 2048,  256, 0x84, 5},
{"16F688"  , FLASH4, 14,  1,  6000,  1,  0, 4096,  256, 0x8c, 5},
{"16C433"  , EPROM , 14,  1,   100, 25,  3, 2048,    0, 0, 0},
{"16C505"  , EPROM , 12,  1,   100,  8, 11, 1024,    0, 0, 0},
{"16C554"  , EPROM , 14,  1,   100, 25,  3,  512,    0, 0, 0},
{"16C556"  , EPROM , 14,  1,   100, 25,  3, 1024,    0, 0, 0},
{"16C558"  , EPROM , 14,  1,   100, 25,  3, 2048,    0, 0, 0},
{"16C61"   , EPROM , 14,  1,   100, 25,  3, 1024,    0, 0, 0},
{"16C62"   , EPROM , 14,  1,   100, 25,  3, 2048,    0, 0, 0},
{"16C620"  , EPROM , 14,  1,   100, 25,  3,  512,    0, 0, 0},
{"16C620A" , EPROM , 14,  1,   100, 25,  3,  512,    0, 0, 0},
{"16C621"  , EPROM , 14,  1,   100, 25,  3, 1024,    0, 0, 0},
{"16C621A" , EPROM , 14,  1,   100, 25,  3, 1024,    0, 0, 0},
{"16C622"  , EPROM , 14,  1,   100, 25,  3, 2048,    0, 0, 0},
{"16C622A" , EPROM , 14,  1,   100, 25,  3, 2048,    0, 0, 0},
{"16C62A"  , EPROM , 14,  1,   100, 25,  3, 2048,    0, 0, 0},
{"16C62B"  , EPROM , 14,  1,   100, 25,  3, 2048,    0, 0, 0},
{"16C63"   , EPROM , 14,  1,   100, 25,  3, 4096,    0, 0, 0},
{"16C63A"  , EPROM , 14,  1,   100, 25,  3, 4096,    0, 0, 0},
{"16C64"   , EPROM , 14,  1,   100, 25,  3, 2048,    0, 0, 0},
{"16C64A"  , EPROM , 14,  1,   100, 25,  3, 2048,    0, 0, 0},
{"16C65"   , EPROM , 14,  1,   100, 25,  3, 4096,    0, 0, 0},
{"16C65A"  , EPROM , 14,  1,   100, 25,  3, 4096,    0, 0, 0},
{"16C65B"  , EPROM , 14,  1,   100, 25,  3, 4096,    0, 0, 0},
{"16C66"   , EPROM , 14,  1,   100, 25,  3, 8192,    0, 0, 0},
{"16C67"   , EPROM , 14,  1,   100, 25,  3, 8192,    0, 0, 0},
{"16C71"   , EPROM , 14,  1,   100, 25,  3, 1024,    0, 0, 0},
{"16C710"  , EPROM , 14,  1,   100, 25,  3,  512,    0, 0, 0},
{"16C711"  , EPROM , 14,  1,   100, 25,  3, 1024,    0, 0, 0},
{"16C712"  , EPROM , 14,  1,   100, 25,  3, 1024,    0, 0, 0},
{"16C716"  , EPROM , 14,  1,   100, 25,  3, 2048,    0, 0, 0},
{"16C72"   , EPROM , 14,  1,   100, 25,  3, 2048,    0, 0, 0},
{"16C72A"  , EPROM , 14,  1,   100, 25,  3, 2048,    0, 0, 0},
{"16C73"   , EPROM , 14,  1,   100, 25,  3, 4096,    0, 0, 0},
{"16C73A"  , EPROM , 14,  1,   100, 25,  3, 4096,    0, 0, 0},
{"16C73B"  , EPROM , 14,  1,   100, 25,  3, 4096,    0, 0, 0},
{"16C74"   , EPROM , 14,  1,   100, 25,  3, 4096,    0, 0, 0},
{"16C745"  , EPROM , 14,  1,   100, 25,  3, 8192,    0, 0, 0},
{"16C74A"  , EPROM , 14,  1,   100, 25,  3, 4096,    0, 0, 0},
{"16C74B"  , EPROM , 14,  1,   100, 25,  3, 4096,    0, 0, 0},
{"16C76"   , EPROM , 14,  1,   100, 25,  3, 8192,    0, 0, 0},
{"16C765"  , EPROM , 14,  1,   100, 25,  3, 8192,    0, 0, 0},
{"16C77"   , EPROM , 14,  1,   100, 25,  3, 8192,    0, 0, 0},
{"16C773"  , EPROM , 14,  1,   100, 25,  3, 4096,    0, 0, 0},
{"16C774"  , EPROM , 14,  1,   100, 25,  3, 4096,    0, 0, 0},
{"16C84"   , FLASH1, 14,  1, 10000,  1,  0, 1024,   64, 0, 0},
{"16C923"  , EPROM , 14,  1,   100, 25,  3, 4096,    0, 0, 0},
{"16C924"  , EPROM , 14,  1,   100, 25,  3, 4096,    0, 0, 0},
{"16C925"  , EPROM , 14,  1,   100, 25,  3, 4096,    0, 0, 0},
{"16C926"  , EPROM , 14,  1,   100, 25,  3, 8192,    0, 0, 0},
{"16CE623" , EPROM , 14,  1,   100, 25,  3,  512,    0, 0, 0},
{"16CE624" , EPROM , 14,  1,   100, 25,  3, 1024,    0, 0, 0},
{"16CE625" , EPROM , 14,  1,   100, 25,  3, 2048,    0, 0, 0},
{"16CR62"  , EPROM , 14,  1,   100, 25,  3, 2048,    0, 0, 0},
{"16CR64"  , EPROM , 14,  1,   100, 25,  3, 2048,    0, 0, 0},
{"16CR84"  , FLASH1, 14,  1, 20000,  1,  0, 1024,   64, 0, 0},
{"16F627"  , FLASH2, 14,  1,  8000,  1,  0, 1024,  128, 0x3d, 5},
{"16F627A" , FLASH4, 14,  1,  6000,  1,  0, 1024,  128, 0x85, 5},
{"16F628"  , FLASH2, 14,  1,  8000,  1,  0, 2048,  128, 0x3e, 5},
{"16F628A" , FLASH4, 14,  1,  6000,  1,  0, 2048,  128, 0x83, 5},
{"16F630"  , FLASH4, 14,  1,  8000,  1,  0, 1024,  128, 0x86, 5},
{"16F648A" , FLASH4, 14,  1,  6000,  1,  0, 4096,  256, 0x88, 5},
{"16F676"  , FLASH4, 14,  1,  8000,  1,  0, 1024,  128, 0x87, 5},
{"16F73"   , FLASH3, 14,  2,  1000,  1,  0, 4096,    0, 0x30, 5},
{"16F74"   , FLASH3, 14,  2,  1000,  1,  0, 4096,    0, 0x31, 5},
{"16F76"   , FLASH3, 14,  2,  1000,  1,  0, 8192,    0, 0x32, 5},
{"16F77"   , FLASH3, 14,  2,  1000,  1,  0, 8192,    0, 0x33, 5},
{"16F818"  , FLASH5, 14,  4,  1000,  1,  0, 1024,  128, 0x4c, 4},
{"16F819"  , FLASH5, 14,  4,  1000,  1,  0, 2048,  256, 0x4e, 4},
{"16F83"   , FLASH1, 14,  1, 20000,  1,  0,  512,   64, 0, 0},
{"16F84"   , FLASH1, 14,  1, 20000,  1,  0, 1024,   64, 0, 0},
{"16F84A"  , FLASH2, 14,  1,  8000,  1,  0, 1024,   64, 0x2b, 5},
{"16F870"  , FLASH2, 14,  1,  4000,  1,  0, 2048,   64, 0x68, 5},
{"16F871"  , FLASH2, 14,  1,  4000,  1,  0, 2048,   64, 0x69, 5},
{"16F872"  , FLASH2, 14,  1,  4000,  1,  0, 2048,   64, 0x47, 5},
{"16F873"  , FLASH2, 14,  1,  4000,  1,  0, 4096,  128, 0x4b, 5},
{"16F873A" , FLASH5, 14,  8,  8000,  1,  0, 4096,  128, 0xe4, 4},
{"16F874"  , FLASH2, 14,  1,  4000,  1,  0, 4096,  128, 0x49, 5},
{"16F874A" , FLASH5, 14,  8,  8000,  1,  0, 4096,  128, 0xe6, 4},
{"16F876"  , FLASH2, 14,  1,  4000,  1,  0, 8192,  256, 0x4f, 5},
{"16F876A" , FLASH5, 14,  8,  8000,  1,  0, 8192,  256, 0xe0, 4},
{"16F877"  , FLASH2, 14,  1,  4000,  1,  0, 8192,  256, 0x4d, 5},
{"16F877A" , FLASH5, 14,  8,  8000,  1,  0, 8192,  256, 0xe2, 4},
{"24C00"   , EEPROM,  8,  8,     0,  0,  0,    0,  128, 0, 0},
{"24C01"   , EEPROM,  8,  8,     0,  0,  0,    0,  128, 0, 0},
{"24C02"   , EEPROM,  8, 16,     0,  0,  0,    0,  256, 0, 0},
{"24C04"   , EEPROM,  8, 16,     0,  0,  0,    0,  512, 0, 0},
{"24C08"   , EEPROM,  8, 16,     0,  0,  0,    0, 1024, 0, 0},
{"24C16"   , EEPROM,  8, 16,     0,  0,  0,    0, 2048, 0, 0},
{"24C32"   , EEPROM,  8, 32,     0,  0,  0,    0, 4096, 0, 0},
{"24C65"   , EEPROM,  8, 32,     0,  0,  0,    0, 8192, 0, 0},
{NULL      ,      0,  0,  0,     0,  0,  0,    0,    0, 0, 0} // terminator
};
#endif
