/*** (C) 1994-> Wim Lewis and others (Credits file)
 *** prog84.c
 ***
 *** these routines use the parallel or serial port
 *** to program a PIC 16C84 in serial mode
 ***/

#include <string.h>
#include <stdlib.h>
#ifdef FASTCPU
#include <unistd.h>
#include <sched.h>
#include <time.h>
#endif
#include "io_ports.h"
#include "prog84.h"

int
    p_power   = -1,  /*  Vpp and/or Vdd power (12-14V / 5V) */
    p_mclr    = -1,  /*  mclr on PIC, 0=reset (0V), often == p_power) */
    p_clock   = -1,
    p_data    = -1,
    p_data_d  = -1,  /* data direction if no open collector is used */
    p_clock_f = -1,
    p_data_f  = -1,
    p_vpp2    = -1,
    p_vpp3    = -1;
int
    /* p_power=0 for cards with C4/C8 connected to 24c16, used by BR875 */
    p_no_power_C4C8 = 0; 
int
    p_loopcnt = -1,
    p_progtype= -1;


struct lp_io lpb;

char *cfg_file = "prog84rc";
char *progpath="" ;

/* If the chips[] array is changed, make sure to change
   the chipstypes in prog84.h : enum {... 
*/

#include "pictypes.h"


chip_info *chip_p = NULL; 

int p84_verbose = 0;
int i2c_debug = 0;
int emulate_write = 0;

unsigned i2cdelay=0;   /* if CPU's get to fast in the future for the 24c16 */

chip_info * get_chip_p(char *chname)  {
  chip_info *chp = chips;
  while ( chp->name && strcasecmp(chp->name, chname) ) 
    chp++; 
  return chp;
}

int set_chip_p(char *chname)  {
  chip_info *ci;

  chip_p = get_chip_p(chname);
  if (chip_p->name == NULL)  {
    int cnt = 0;
    /* print list of chips available */
    ci = chips;
    fprintf(stderr,"Supported chips:\n");
    while (ci->name) {
      fprintf(stderr," %s",ci->name); 
      if ( ++cnt % 8 == 0) 
	fprintf(stderr,"\n");
      ci++;
    }
    fprintf(stderr,"\n");
    chip_p = NULL;
    return 0;
  }
  //fprintf(stderr," %s \n",chip_p->name);
  return 1;
}

void printListSupported() {
  int i=0;
  
  fprintf(stderr,"List of supported devices:\n\n"
                 "Chip            Program     Data\n"
                 "--------------------------------\n");
  while (chips[i].name) 
  {
      fprintf(stderr,"%-15s  %6d   %6d\n",chips[i].name,
        chips[i].prog_size, chips[i].eeprom_size);
      i++;
  }
}


static int file_exist(char *s) {
  FILE *f;
  if (s==NULL) return 0;
  if ( (f=fopen(s,"r"))==NULL) return 0;
  fclose(f);
  return 1;
}

/* set program path from argv */
void set_progpath(char *p) {
  char *pr;
  pr=strrchr(p,'/');
  if (pr!=NULL) {  
	int  l;
	l=strlen(p)-strlen(pr);
	progpath=calloc(l+2,sizeof(char));
	strncpy(progpath,p,l+1);
	pr=calloc(l+2+strlen(cfg_file),sizeof(char));
	strcpy(pr,progpath);
	strcat(pr,cfg_file);
  } else pr="";
 /* find lp_cfg:  
    1st.: currentdir; 2nd.: $HOME/.prog84rc; 3rd.:"progpath"/prog84rc */
  if (verbose > 1) fprintf(stderr,"cfg file, try : %s\n",cfg_file);
  if ( ! file_exist(cfg_file) ) {
#ifdef __MSDOS__
    if ( (strlen(pr)>0) && file_exist(pr)) cfg_file=pr;
#else
    char *home;
    char *cf;
    home = getenv("HOME");
    if (home != NULL) {
	cf=calloc(strlen(cfg_file)+strlen(home)+3,sizeof(char));
	strcpy(cf,home); strcat(cf,"/.");
	strcat(cf,cfg_file);
    } else cf=pr;
    if (file_exist(cf)) cfg_file=cf;
    else {
      if (file_exist(pr)) cfg_file=pr;
    }
#endif
  }

  /* read the configuration file */
  if (lpb_cfg_read(cfg_file /*, config_names*/, &(lpb.io_usage))) {
	fprintf(stderr, "error reading cfg file %s\n", cfg_file);
	exit(1);
  }

  /* BR875 hack from Frank Damgaard */
  if ( p_no_power_C4C8 > 0 ) {
	// BR875 has some timing/signal level problems
	// adding the number of ports to read increases delay
	// seems to work
        lpb_set_extra_delay(&(lpb.io_usage));
  }
}

/***
 *** poweron the pic and enter programming mode
 ***/
void power_and_reset()
{
  int cap_charge=1;

  /* clear power, if it still on from previos call */
  SET(mclr, 0);
  if (p_power!= -1)  SET(power, 0);
  SET(clock, 1);
  if ( p_data_d != -1) 
    SET(data_d, 1);
  SET(data, 0);
  if (p_vpp2!=-1) SET(vpp2,0);
  if (p_vpp3!=-1) SET(vpp3,0);
  FLUSH;
  usleep(200000UL); /* 0,2 s is hopefully enough to reset programmer */
  
  /* set power to pic-programmer */
  SET(mclr, 0);
  if (chip_p && chip_p->type==EEPROM && p_no_power_C4C8>0 && p_power != -1) {
    // eeprom wanted  :
    SET(power,0);   /* BR875 needs Vpp=0 for eeprom. */
    if (p84_verbose) fprintf(stderr,"BR875: Vpp=0\n");
  } else {
    if (p_power!= -1)
      SET(power, 1);	/* the ludpipo serial programmer needs this... */
    if (p_vpp2!= -1) SET(vpp2, 1);
    if (p_vpp3!= -1) SET(vpp3, 1);
  }
  SET(clock, 0);
  SET(data, 0);
  FLUSH;
  if (p84_verbose) fprintf(stderr,"Charging capacitor (%d sec.)...\n",cap_charge);
  sleep(cap_charge); /* wait quite a bit to charge capacitor */
  SET(mclr, 0);
  FLUSH;
  
  usleep(50000UL);
  if (! (chip_p && chip_p->type==EEPROM && p_no_power_C4C8>0)) {
    // eeprom wanted  :
    SET(mclr, 1);
    FLUSH;
  }
  usleep(50000UL);
}

/* Read device ID */
int get_pictype() 
{
  unsigned int i, devid;
  chip_info *ci = chips;

  power_and_reset();
  /* dummy load configuration command to get to cfg memory */
  loadConfiguration(0x3FFF);
  for (i=0; i<6; i++) 
    incAddr();
  devid = readProg();
  if( verbose )
    fprintf(stderr, "Device ID = 0x%x\n", devid );
  while(ci->name)
  {
    if(ci->dev_id && ci->dev_id == (devid >> ci->rev_bits ))
    {
       fprintf(stderr, "Autodetected %s chip\n", ci->name );
       chip_p = ci;
       return 1;
    }
    ci++;
  }
  return 0;
}

int test_chip() 
{
  int	r;

  /* try to autodetect the PIC type */
  if ( chip_p == NULL )
    get_pictype(  );

  if ( chip_p && chip_p->type == EEPROM )
  {
    // eeprom wanted on BR875, so turn on eeprom logic:
    if (p_no_power_C4C8>0 && p_power != -1) {
      SET(power,0); FLUSH ;
    }
    // verify EEPROM presence
    i2c_start();
    r = i2c_sendbyte( EE_CTRL_WR );
    i2c_stop();
    if( r )
      fprintf(stderr,"EEPROM present\n");
    else
      return 0;
    }
  
  if( chip_p )  {
    fprintf(stderr,"Using %s chip, capacity %d data, %d code\n",
	    chip_p->name, chip_p->eeprom_size, chip_p->prog_size);
    return 1;
  }
  return 0;
}

int testHW()
{
    int i,j;     /* utility variables */
    int ret[4];  /* array of test results */
    int m;       /* merged test results */

    power_and_reset();

    if (p_mclr == -1 || p_clock == -1 || p_data == -1 || p_data_f == -1) {
	fprintf(stderr, 
	    "Error: I need control over MCLR, clock, and data,\n"
	    "       and I need to be able to read data back.\n");
	return 1;
    }


    if (p84_verbose) {
	fprintf(stderr,"[testing loopback   ");
	fflush(stderr);
    }

    if ( p_data_d != -1) 
  	SET(data_d, 1);
    for (i=0; i<4; i++) {
	SET(clock, i&1);
	SET(data, i&2);
	FLUSH;
	usleep(100UL); /* here I can be slow, no problem */
	/* 2001-01-09 UPS:  added REFRESH input , 
	   since FLUSH reads data to soon, and GET only reads buffer */
	REFRESH;
	j = GET(data_f) ? 2 : 0;
	if (p_clock_f != -1)
	    j |= GET(clock_f)? 1 : 0;
	else
	    j |= i & 1;  /* fake it if we don't have it */
	ret[i] = j;
    }
    SET(clock, 0);
    SET(data, 0);
    FLUSH;

    if (p84_verbose)
	fprintf(stderr,"  ]\n");

    /***
     *** analyze the test results. We put our four 2-bit responses
     *** into an int, one per nybble so we can use handy hex notation.
     ***/

    m = (((((ret[3] << 4) | ret[2]) << 4) | ret[1]) << 4) | ret[0];

    if(m == 0x3210)	/*** this is what we expect ***/
	return 0;

    fprintf(stderr, "Bad test results!\n  [data, clock]->[data_f, clock_f]:");
    for(i = 0; i < 4; i ++)
	fprintf(stderr, " %d%d->%d%d", (i>>1)&1, i&1,
		(ret[i]>>1)&1, (ret[i])&1);
    fprintf(stderr,"\n");

    if (p_clock_f == -1)
	fprintf(stderr, "(clock_f simulated since I don't actually have it)\n");

    if (m == 0x3120)
	fprintf(stderr, "clock and data reversed?\n"
	    " (could be either the clock & data lines, or the sense lines)\n"
	);

    switch(m & 0x1111) {
    case 0x0000:
	fprintf(stderr, "clock_f stuck low?\n");
	break;
    case 0x0101:
	fprintf(stderr, "clock_f inverted?\n");
	break;
    case 0x1111:
	fprintf(stderr, "clock_f stuck high?\n");
	break;
    }

    switch(m & 0x2222) {
    case 0x0000:
	fprintf(stderr,"data_f stuck low?\n");
	break;
    case 0x0022:
	fprintf(stderr,"data_f inverted?\n");
	break;
    case 0x2222:
	fprintf(stderr,"data_f stuck high?\n");
	break;
    }

    /***
     *** here I evidently have a failure, and should return 1. However,
     *** I keep on trying in the hope it is a temporary thing.
     ***/

    return 0;
}

/*** 
 *** Pipe some bits through the 16C84. Returns the values read from
 *** the data pin at the appropriate point in the cycle for reading
 *** data mem.  In a write, will return 'bits'.  (Unless, of course,
 *** something's wrong.)
 *** When reading set bits=0x7ffe
 ***/

#if defined( __MSDOS__ ) 
int loopd1=2,loopd2=200;
#endif

unsigned int pipeBits(unsigned int bits, int count)
{
    unsigned int ret = 0;
    int bit;

    for (bit=0; bit<count; bit++) {
	SET(clock, 1);
	SET(data, bits&1);
	FLUSH;
	/* Minumum 100 ns. delay is needed when programming PIC's
	   specifications require min. 100ns clock low and 100ns clock low.
	   Data must be valid > 80ns before clock transition 1->0
	   FLUSH ( lpb_flush() , lp_refresh() ) usually
	   gives adequate delay on linux/frebsd systems */
	SLEEP100ns(loopd1); /* 0.2 us */
	SET(clock, 0);
	FLUSH;   /* <- loads data in PIC , & reads input & sets data direction */
	ret |= (GET(data_f)? 1 : 0 ) << bit;
	if (p84_verbose > 1)
	    fprintf(stderr,". bit %2d: send %d get %d\n",
		bit, bits&1, GET(data_f)?1:0);

	/* minimum 100ns delay needed : */
	SLEEP100ns(loopd1); /* 0.2us */
	bits >>= 1;
    }
    SET(data, 1); /* the HI-Z state */
    FLUSH;
    /* required guard time of at least 1 usec.
       seems for fast cpu + dos/win at least 1ms is needed !? 
       .... faulty sleep/delay algorithms in DOS */
    SLEEP100ns(loopd2); /* 2.0 us */
    return ret;
}

void xPipeBits(unsigned int bits, int count)
{
    unsigned int ret;

    ret = pipeBits(bits, count);
    if (ret != bits) {
      fprintf(stderr,
	      "ERROR: sent %d bits: %d (0x%x), got %d (0x%x)\n"
	      "Resetting & powering down.\n",
	      count, bits, bits, ret, ret);
      progShutdown();
      exit(2);
    }
}

/* A generic command-plus-14-bits-to-write call */
void genericWrite(int cmd, unsigned int data)
     /* 6 bits of command, 14 bits of data */
{
    xPipeBits(cmd, 6);		  	/* Send the command */
    xPipeBits((data << 1) & 0x7FFE, 16);/* send the data */
}

/* A generic command-plus-14-bits-to-read call */
unsigned int genericRead(int cmd)
{
    unsigned int ret;

    xPipeBits(cmd, 6);			/* send the command */
    delay_ns(1000UL);		/* 1 usec delay between cmd and data */
    if ( p_data_d != -1) 
      SET(data_d, 0);		/* High-Z data */
    ret = pipeBits(0x7FFE, 16);		/* read the result */
    if ( p_data_d != -1) 
      SET(data_d, 1);
    return((ret & 0x7FFE) >> 1);/* remove guard bits */
}

/* use usleep instead if the delay is > 50ms */
void delay_ns(unsigned long dly) 
{
#ifdef FASTCPU
  struct timespec time;
  {
    time.tv_sec = 0;
    time.tv_nsec = dly;
    nanosleep(&time, NULL);
  }
#else
  int i;
  if (p_loopcnt>0)
    for(i = 0; i < p_loopcnt; i++)
      while( dly-- )
	;
#endif
}

void progSetup() {
#ifdef FASTCPU
  /* enable real-time scheduling so we can use short delays with nanosleep */
#ifdef _POSIX_PRIORITY_SCHEDULING
  {
  struct sched_param mysched;
    int	err;
  
    if ( sched_getparam( 0, &mysched ) != 0 )
      perror( "" );
    else
    {
  mysched.sched_priority = 90;
      err = sched_setscheduler(0, SCHED_FIFO, &mysched);
      if( err != 0 )
      {
        fprintf (stderr,"sched_setschedular returned: %d\n", err );
        perror( "" );
      }
    }
  }
#endif
#endif
  
  /* read the configuration file : moved to set_prog_path() */
  /* but print debug info if verbose : */
  if (p84_verbose)
    lpb_dump_names( &lpb.io_usage /*.hw_type*/ /*, config_names*/);
  
  /* open /dev/port */
  lpb_open(&lpb);
  
  /* check that the programmer's not totally dead */
  if (testHW()) {
    fprintf(stderr,"\nProgrammer is not responding, aborting.\n");
    exit(1);
  }
  if( ! test_chip() )  {
    fprintf(stderr,"\nChip was neither autodetected or specified.\n");
    exit(1);
  }
}

void progShutdown()
{
    if (verbose) fprintf(stderr,"...closing\n");
    SET(mclr, 0);
    FLUSH;
    usleep(10000UL);	/* shutdown... */
    if (p_vpp3!=1) SET(vpp3, 0);
    if (p_vpp2!=1) SET(vpp2, 0);
    if (p_power!=1) SET(power, 0);
    SET(clock, 0);
    SET(data, 0);
    FLUSH;
    lpb_close(&lpb);

#ifdef FASTCPU
    /* disable real-time scheduling */
#ifdef _POSIX_PRIORITY_SCHEDULING
    {
      struct sched_param mysched;
      int	err;
  
      if ( sched_getparam( 0, &mysched ) != 0 )
        perror( "" );
      else
      {
        mysched.sched_priority = 0;
        err = sched_setscheduler(0, SCHED_OTHER, &mysched);
        if( err != 0 )
        {
          fprintf (stderr,"sched_setschedular returned: %d\n", err );
          perror( "" );
        }
      }
    }
#endif
#endif
}

/* 990103 Frank Damgaard , for eeprom :
   Partially borrowed from i2c driver in "bttv" package,
   and ideas from serp-0.5.tgz
   Datasheet for 24c16 from www.microchip.com was a good help too :)
*/

void i2c_out(WORD c, WORD d) {
  SET(clock,c); SET(data,d); FLUSH;
  /* I2C_DEBUG(fprintf(stderr,"%c",d?'1':'0')); */
#ifdef FASTCPU
  delay_ns(5000UL);
#else
  usleep(5UL);  
#endif
}

void i2c_start()
{ /* the 1->0 transition while clock=1 => start */
        I2C_SET(0,1);
        I2C_SET(1,1);
        I2C_SET(1,0);
        I2C_SET(0,0);
        I2C_DEBUG(fprintf(stderr,"S ")) ;
}

void i2c_stop()
{ /* the 0->1 transition while clock=1 => stop */
        I2C_SET(0,0);
        I2C_SET(1,0);
        I2C_SET(1,1);
        I2C_SET(0,1);
        I2C_DEBUG(fprintf(stderr,"P\n"));
}

void i2c_data(WORD dat){ 
  /* data is clocked during clock=1 */
        I2C_SET(0,dat);
        I2C_SET(1,dat);
        I2C_SET(0,dat);
}

int i2c_rdack()
{ /* 
  return:  0  if no acknowledge
           1  if acknowledge 
*/
     int rack;
     if ( p_data_d != -1) 
       SET(data_d, 0);
     I2C_SET(0,1);
     I2C_SET(1,1);
	REFRESH;
        rack = ! I2C_GET();
     if ( p_data_d != -1) 
       SET(data_d, 1);
     I2C_SET(0,1);
        return rack; 
}

int i2c_sendbyte(unsigned char dat) {
        int i, ack;
    
        I2C_SET(0,0);
        for (i=7; i>=0; i--) i2c_data(dat&(1<<i));
        ack=i2c_rdack();
        I2C_DEBUG(fprintf(stderr," %02x%c ",(int)dat,ack?'+':'-'));
        return ack;
}

void i2c_sendctrl(unsigned char com, WORD adr) {
  unsigned char ctrl=0;
  unsigned int mask;
  
  mask = (chip_p->eeprom_size - 1) >> 8;
  /* 2404/2408/2416 uses 1-3 bit of control word for A10-A9-A8 */
  if( mask && mask == (mask & 7) )
  {
    mask <<= 1;
    ctrl &= ~mask;
    ctrl |= adr>>7 & mask;
  }
  ctrl &= 0x0e;
  ctrl |= com;
  i2c_sendbyte(ctrl);
}

int i2c_waitack(unsigned int t) {
/* block write and byte write need up to 5 miliseconds wait time. */
/* t is max time in microseconds */
  int ack;
  
  t /= 160; // 160 us is the length of a verify ack cycle
  // PC timer based resolution is 18 ms anyways
  if( !t ) t = 1;
  while(t--)
  {
	i2c_start();
    ack=i2c_sendbyte(EE_CTRL_WR);
    i2c_stop();
    if (ack) return 1;
  }
  return 0;
}

unsigned char i2c_readbyte(int ack) {
        int i;
        unsigned char dat=0;
    
        if ( p_data_d != -1) 
          SET(data_d, 0);
        I2C_SET(0,1);
        for (i=7; i>=0; i--) {
                I2C_SET(1,1);
		REFRESH; 
                if (I2C_GET())
                        dat |= (1<<i);
                I2C_SET(0,1);
	}
        if ( p_data_d != -1) 
          SET(data_d, 1);
	i2c_data(ack); 
	I2C_DEBUG(fprintf(stderr,"=%02x%c ",(int)dat,ack?'-':'+'));
        return dat;
}

void i2c_sendaddr(WORD adr) {
  i2c_sendctrl(EE_CTRL_WR,adr);
  /* 2432/2465 uses an extra address word */
  if (chip_p->eeprom_size >= 4096 ) i2c_sendbyte((adr>>8)&0x1f);
  i2c_sendbyte(adr&0xff);
  I2C_DEBUG(fprintf(stderr,"addr=%04x ",(int)adr));
}

void i2c_seqread(WORD adr, unsigned char *data, int len) {
  int i;
  i2c_start();
  i2c_sendaddr(adr);
  i2c_start();
  i2c_sendctrl(EE_CTRL_RD, adr);
  for (i=0; i<len-1; i++) {
	data[i]=i2c_readbyte(I2C_ACK);
  }
  data[len-1]=i2c_readbyte(I2C_NOACK);
  i2c_stop();
}

unsigned char i2c_read(WORD adr) {
  unsigned char data;
  i2c_start();
  i2c_sendaddr(adr);
  i2c_start();
  i2c_sendctrl(EE_CTRL_RD, adr);
  data=i2c_readbyte(I2C_NOACK);
  I2C_DEBUG(fprintf(stderr," data=%02x ",(int)data));
  i2c_stop();
  return data;
}


void i2c_pagewrite(WORD adr, unsigned char *dat, WORD PageSize) {
/* Must do an i2c_waitack() after i2c_pagewrite()! */
/* *dat must hold not more than 16 words (unsigned char) !! */
/* 24c16/c08 only has 16 byte storage so pagewrite() only writes 16 bytes*/

  int i;
  i2c_start();
  i2c_sendaddr(adr);
  for (i=0; i<PageSize; i++) i2c_sendbyte(dat[i]);
  i2c_stop();
}

void i2c_write(WORD adr, WORD data) {
/* Must do an i2c_waitack() after i2c_write()! */
  i2c_start();
  i2c_sendaddr(adr);
  i2c_sendbyte(data);
  i2c_stop();
}

